import pandas as pd
from typing import List
from random import Random
from .add_chunk import add_chunk


def add_row(csv: pd.DataFrame, values: List, iloc: int = None, state: Random = None):
    if iloc is None:
        iloc = state.randint(0, len(csv))
    return add_chunk(csv, pd.DataFrame(dict(zip(csv.columns, values)), index=[iloc]), iloc)


import pandas as pd
import numpy as np

def add_chunk(csv: pd.DataFrame, chunk: pd.DataFrame, iloc: int):
    if chunk.shape[1] < csv.shape[1]:
        old_chunk = chunk
        chunk = pd.DataFrame(np.nan, columns=csv.columns, index=old_chunk.index, dtype="object")
        chunk.values[:old_chunk.shape[0], :old_chunk.shape[1]] = old_chunk.values
    return pd.concat([
        csv.iloc[:iloc],
        chunk,
        csv.iloc[iloc:]
    ]).reset_index(drop=True)


import pandas as pd
import numpy as np
from random import Random
from tqdm.auto import tqdm


def add_empty_columns(csv: pd.DataFrame, state: Random, verbose: bool) -> pd.DataFrame:
    """Add random empty columns"""
    columns_number = state.randint(0, len(csv.columns))
    for number in tqdm(range(columns_number), desc="Adding empty columns", disable=not verbose or columns_number==0):
        loc = state.randint(0, len(csv.columns))
        csv.insert(
            loc=loc,
            column="{column}{separator}{number}".format(
                column=state.choice(csv.columns),
                separator=state.choice(("", " ", ".", "_", "-")),
                number=number
            ),
            value=np.nan
        )
    return csv



import pandas as pd
from random import Random
from .utils import (
    add_empty_columns, add_empty_rows, add_duplicate_schema,
    add_empty_padding, add_nan_like_artefacts, add_random_spaces,
    add_satellites
)


def uglify(
    csv: pd.DataFrame,
    # path: str = None,
    empty_columns: bool = True,
    empty_rows: bool = True,
    duplicate_schema: bool = True,
    empty_padding: bool = True,
    nan_like_artefacts: bool = True,
    # random_separator: bool = True,
    satellite_artefacts: bool = True,
    random_spaces: bool = True,
    verbose: bool = True,
    seed: int = 42
):
    """Return or saves to given path an uglified version of the given DataFrame.

    The procedure is non-destructive,
    meaning all infomation should remain retrievable
    using a valid reconstruction method.

    Parameters
    ---------------------------------
    csv:pd.DataFrame,
        The dataframe containing the CSV to uglify.
    path:str=None,
        Path where to save the CSV.
    empty_columns:bool=True,
        Whetever to introduce empty columns (with header).
    empty_padding:bool=True,
        Whetever to introduce padding around the data.
    random_separator:bool=True,
        Whetever to use a random separator when saving the file.
    satellite_artefacts:bool=True,
        Whetever to add satellite text around the central table.
        This is useful to simulate when you can expect for
        random scribbling around your CSV data.
        The satellites may be added on the top or bottom.
    random_spaces:bool=True,
        Whetever to add random spaces to the file values.
    empty_rows:bool=True,
        Whetever to introduce random blank rows.
    duplicate_schema:bool=True,
        Whetever to duplicate the schema rows within the data.
    verbose: bool = True,
        Whetever to show the loading bars.
    seed:int=42
        The random seed to use to make reproducible the random aspects.

    Returns
    --------------------------------
    The uglified dataframe.
    """
    state = Random(seed)

    if duplicate_schema:
        csv = add_duplicate_schema(csv, state, verbose=verbose)

    if random_spaces:
        csv = add_random_spaces(csv, state)

    if empty_columns:
        csv = add_empty_columns(csv.copy(), state, verbose=verbose)

    if empty_rows:
        csv = add_empty_rows(csv, state, verbose=verbose)

    if empty_padding:
        csv = add_empty_padding(csv, state)

    if satellite_artefacts:
        csv = add_satellites(csv, state)

    if nan_like_artefacts:
        csv = add_nan_like_artefacts(csv, state)

    return csv



from .add_empty_columns import add_empty_columns
from .add_empty_rows import add_empty_rows
from .add_duplicate_schema import add_duplicate_schema
from .add_empty_padding import add_empty_padding
from .add_nan_like_artefacts import add_nan_like_artefacts
from .add_random_spaces import add_random_spaces
from .add_satellites import add_satellites

__all__ = [
    "add_empty_columns",
    "add_empty_rows",
    "add_duplicate_schema",
    "add_empty_padding",
    "add_nan_like_artefacts",
    "add_random_spaces",
    "add_satellites"
]


import pandas as pd
from random import Random


def random_space(state: Random):
    k = state.randint(1, 10)
    return "".join(state.choices([" ", "\n", "\n\r"], [0.8, 0.15, 0.05], k=k))


def add_random_spaces(csv: pd.DataFrame, state: Random) -> pd.DataFrame:
    return csv.applymap(lambda x: "{s1}{x}{s2}".format(
        s1=random_space(state),
        x=str(x).replace(" ", random_space(state)),
        s2=random_space(state)
    ) if not pd.isna(x) else x)



import pandas as pd
import numpy as np
from random import Random
from .add_row import add_row


def add_empty_padding(csv: pd.DataFrame, state: Random) -> pd.DataFrame:
    csv = add_row(csv, csv.columns, 0)
    w = len(csv.columns)
    h = len(csv)
    columns_number = state.randint(w, w*2)
    rows_number = state.randint(h, h*2)
    empties = np.empty((rows_number, columns_number), dtype='O')
    empties.fill(np.nan)
    if columns_number-w != 0:
        start_x = state.randint(0, columns_number-w)
    else:
        start_x = 0
    if rows_number-h != 0:
        start_y = state.randint(0, rows_number-h)
    else:
        start_y = 0

    empties[start_y:start_y+h, start_x:start_x+w] = csv.values

    return pd.DataFrame(
        empties, columns=list(range(empties.shape[1]))
    )



import pandas as pd
import numpy as np
from random import Random


def random_nan(state: Random):
    if state.randint(0, 1) == 1:
        return state.choice([np.nan, 0, "#RIF!"])
    n = state.randint(1, 10)
    return state.choice([".", "-", "_", "/", " ", "\n", "\n\r"])*n


def add_nan_like_artefacts(csv: pd.DataFrame, state: Random) -> pd.DataFrame:
    return csv.applymap(lambda x: random_nan(state) if pd.isna(x) else x)



from .add_row import add_row
import pandas as pd
from random import Random
from tqdm.auto import tqdm


def add_duplicate_schema(csv: pd.DataFrame, state: Random, verbose:bool) -> pd.DataFrame:
    """Duplicate the schema into random new rows."""
    schema_number = state.randint(0, max(1, len(csv)//2))
    for _ in tqdm(range(schema_number), desc="Adding duplicated schemas", disable=not verbose):
        csv = add_row(csv, csv.columns, state=state)
    return csv



from .add_row import add_row
import pandas as pd
import numpy as np
from random import Random
from tqdm.auto import tqdm


def add_empty_rows(csv: pd.DataFrame, state: Random, verbose:bool) -> pd.DataFrame:
    """Add random empty rows"""
    rows_number = state.randint(0, max(1, len(csv)//2))
    empties = np.empty(len(csv.columns))
    empties.fill(np.nan)
    for _ in tqdm(range(rows_number), desc="Adding empty rows", disable=not verbose):
        csv = add_row(csv, empties, state=state)
    return csv



import pandas as pd
import os
from random import Random
from glob import glob
from .add_chunk import add_chunk

def random_satellite(state: Random):
    satellites = glob("{}/satellites/*.csv".format(os.path.dirname(os.path.abspath(__file__))))
    return pd.read_csv(state.choice(satellites), header=None)


def add_satellites(csv: pd.DataFrame, state: Random) -> pd.DataFrame:
    if state.randint(0, 1) == 1:
        csv = add_chunk(csv, random_satellite(state), 0)
    if state.randint(0, 1) == 1:
        csv = add_chunk(csv, random_satellite(state), len(csv))
    return csv